@@ -1,7 +1,8 @@
 /*
  *      gb.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,7 +18,11 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: gb.c 639 2006-07-26 17:02:16Z eht16 $
+ * $Id: gb.c 1329 2007-02-24 11:41:56Z ntrel $
+ */
+
+/*
+ * GTK-Bandit.
  */
 
 #include <stdlib.h>
@@ -67,7 +72,7 @@ static void create_window(void)
 	GtkWidget *button5;
 
 	gb_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_title(GTK_WINDOW(gb_window), "Happy Eastern!");
+	gtk_window_set_title(GTK_WINDOW(gb_window), "Happy Easter!");
 	gtk_window_set_resizable(GTK_WINDOW(gb_window), FALSE);
 	gtk_window_set_position(GTK_WINDOW(gb_window), GTK_WIN_POS_CENTER);
 	gtk_window_set_destroy_with_parent(GTK_WINDOW(gb_window), TRUE);
@@ -1324,7 +1329,7 @@ static gboolean gb_on_key_pressed(GtkWidget *widget, GdkEventKey *event, gpointe
 		{
 			text[4] = 'y';
 			text[5] = '\0';
-			if (utils_strcmp(text, "geany")) gb_start_easteregg();
+			if (utils_str_equal(text, "geany")) gb_start_easteregg();
 			return TRUE;
 			break;
 		}
