@@ -1,7 +1,8 @@
 /*
  *      images.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,6 +20,9 @@
  *
  */
 
+/*
+ * Inline image data.
+ */
 
 /* GdkPixbuf RGBA C-Source image dump */
 #include <gtk/gtk.h>
@@ -218,25 +222,24 @@ static const guint8 close_small_inline[] =
 { ""
   /* Pixbuf magic (0x47646b50) */
   "GdkP"
-  /* length: header (24) + pixel_data (196) */
-  "\0\0\0\334"
-  /* pixdata_type (0x1010002) */
-  "\1\1\0\2"
-  /* rowstride (28) */
-  "\0\0\0\34"
-  /* width (7) */
-  "\0\0\0\7"
-  /* height (7) */
-  "\0\0\0\7"
+  /* length: header (24) + pixel_data (171) */
+  "\0\0\0\303"
+  /* pixdata_type (0x2010002) */
+  "\2\1\0\2"
+  /* rowstride (40) */
+  "\0\0\0("
+  /* width (10) */
+  "\0\0\0\12"
+  /* height (10) */
+  "\0\0\0\12"
   /* pixel_data: */
-  "\0\0\0\0\0\0\0\377\0\0\0\0\0\0\0\0\0\0\0\377\0\0\0\377\0\0\0\0\0\0\0"
-  "\0\0\0\0\377\0\0\0\377\0\0\0\0\0\0\0\377\0\0\0\377\0\0\0\0\0\0\0\0\0"
-  "\0\0\0\0\0\0\377\0\0\0\377\0\0\0\377\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0"
-  "\0\0\0\377\0\0\0\377\0\0\0\377\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0"
-  "\377\0\0\0\377\0\0\0\377\0\0\0\377\0\0\0\0\0\0\0\0\0\0\0\377\0\0\0\377"
-  "\0\0\0\0\0\0\0\377\0\0\0\377\0\0\0\377\0\0\0\0\0\0\0\377\0\0\0\0\0\0"
-  "\0\0\0\0\0\0\0\0\0\377\0\0\0\377"};
-
+  "\2\0\0\0\10\0\0\0\35\206\0\0\0\0\202\0\0\0\15\1\0\0\0\1\202\0\0\0\377"
+  "\204\0\0\0\0\202\0\0\0\377\2\0\0\0\31\0\0\0\0\203\0\0\0\377\202\0\0\0"
+  "\0\203\0\0\0\377\203\0\0\0\0\206\0\0\0\377\205\0\0\0\0\204\0\0\0\377"
+  "\206\0\0\0\0\204\0\0\0\377\205\0\0\0\0\206\0\0\0\377\203\0\0\0\0\203"
+  "\0\0\0\377\202\0\0\0\0\203\0\0\0\377\2\0\0\0\0\0\0\0\13\202\0\0\0\377"
+  "\204\0\0\0\0\202\0\0\0\377\202\0\0\0\22\1\0\0\0\11\206\0\0\0\0\2\0\0"
+  "\0\21\0\0\0\40"};
 
 /* GdkPixbuf RGBA C-Source image dump */
 
