@@ -1,7 +1,8 @@
 /*
  *      main.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,26 +18,29 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: main.c 685 2006-08-09 11:06:51Z ntrel $
+ * $Id: main.c 1503 2007-05-04 15:33:07Z eht16 $
  */
 
+/*
+ * Program initialization and cleanup.
+ */
 
 #include <signal.h>
 #include <time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <unistd.h>
-#include <fcntl.h>
 #include <errno.h>
 #include <string.h>
 #include <stdlib.h>
 
 #include "geany.h"
 
+#include "main.h"
 #include "interface.h"
 #include "support.h"
 #include "callbacks.h"
 
+#include "ui_utils.h"
 #include "utils.h"
 #include "document.h"
 #include "keyfile.h"
@@ -49,6 +53,16 @@
 #include "notebook.h"
 #include "keybindings.h"
 #include "sci_cb.h"
+#include "search.h"
+#include "build.h"
+#include "highlighting.h"
+#include "symbols.h"
+#include "project.h"
+#include "tools.h"
+
+#ifdef HAVE_SOCKET
+# include "socket.h"
+#endif
 
 #ifdef HAVE_VTE
 # include "vte.h"
@@ -56,10 +70,10 @@
 
 #define N_(String) (String)
 
-#ifdef HAVE_FIFO
-static gchar fifo_name[512];
-static gboolean ignore_fifo = FALSE;
-#endif
+
+
+CommandLineOptions cl_options;	// fields initialised in parse_command_line_options
+
 static gboolean debug_mode = FALSE;
 static gboolean ignore_global_tags = FALSE;
 static gboolean no_msgwin = FALSE;
@@ -69,23 +83,31 @@ static gchar *alternate_config = NULL;
 static gboolean no_vte = FALSE;
 static gchar *lib_vte = NULL;
 #endif
+#ifdef HAVE_SOCKET
+static gboolean ignore_socket = FALSE;
+#endif
 static gboolean generate_datafiles = FALSE;
+static gboolean generate_tags = FALSE;
 
 static GOptionEntry entries[] =
 {
-	{ "debug", 'd', 0, G_OPTION_ARG_NONE, &debug_mode, N_("runs in debug mode (means being verbose)"), NULL },
-	{ "no-ctags", 'n', 0, G_OPTION_ARG_NONE, &ignore_global_tags, N_("don't load auto completion data (see documentation)"), NULL },
-#ifdef HAVE_FIFO
-	{ "no-pipe", 'p', 0, G_OPTION_ARG_NONE, &ignore_fifo, N_("don't open files in a running instance, force opening a new instance"), NULL },
-#endif
+	{ "column", 0, 0, G_OPTION_ARG_INT, &cl_options.goto_column, N_("set initial column number for the first opened file (useful in conjunction with --line)"), NULL },
 	{ "config", 'c', 0, G_OPTION_ARG_FILENAME, &alternate_config, N_("use an alternate configuration directory"), NULL },
+	{ "debug", 'd', 0, G_OPTION_ARG_NONE, &debug_mode, N_("runs in debug mode (means being verbose)"), NULL },
+	{ "generate-tags", 'g', 0, G_OPTION_ARG_NONE, &generate_tags, N_("generate global tags file (see documentation)"), NULL },
+	{ "generate-data-files", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &generate_datafiles, "", NULL },
+#ifdef HAVE_SOCKET
+	{ "new-instance", 'i', 0, G_OPTION_ARG_NONE, &ignore_socket, N_("don't open files in a running instance, force opening a new instance"), NULL },
+#endif
+	{ "line", 'l', 0, G_OPTION_ARG_INT, &cl_options.goto_line, N_("set initial line number for the first opened file"), NULL },
 	{ "no-msgwin", 'm', 0, G_OPTION_ARG_NONE, &no_msgwin, N_("don't show message window at startup"), NULL },
+	{ "no-ctags", 'n', 0, G_OPTION_ARG_NONE, &ignore_global_tags, N_("don't load auto completion data (see documentation)"), NULL },
+	{ "no-session", 's', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &cl_options.load_session, N_("don't load the previous session's files"), NULL },
 #ifdef HAVE_VTE
 	{ "no-terminal", 't', 0, G_OPTION_ARG_NONE, &no_vte, N_("don't load terminal support"), NULL },
-	{ "vte-lib", 'l', 0, G_OPTION_ARG_FILENAME, &lib_vte, N_("filename of libvte.so"), NULL },
+	{ "vte-lib", 0, 0, G_OPTION_ARG_FILENAME, &lib_vte, N_("filename of libvte.so"), NULL },
 #endif
 	{ "version", 'v', 0, G_OPTION_ARG_NONE, &show_version, N_("show version and exit"), NULL },
-	{ "generate-data-files", 'g', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &generate_datafiles, "", NULL },
 	{ NULL, 0, 0, 0, NULL, NULL, NULL }
 };
 
@@ -110,7 +132,7 @@ void geany_debug(gchar const *format, ...)
  * (all the following code is not perfect but it works for the moment) */
 static void apply_settings(void)
 {
-	utils_update_fold_items();
+	ui_update_fold_items();
 
 	// toolbar, message window and sidebar are by default visible, so don't change it if it is true
 	if (! app->toolbar_visible)
@@ -120,7 +142,7 @@ static void apply_settings(void)
 		gtk_widget_hide(app->toolbar);
 		app->ignore_callback = FALSE;
 	}
-	if (! app->msgwindow_visible || no_msgwin)
+	if (! app->msgwindow_visible)
 	{
 		app->ignore_callback = TRUE;
 		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget(app->window, "menu_show_messages_window1")), FALSE);
@@ -133,7 +155,7 @@ static void apply_settings(void)
 		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget(app->window, "menu_show_sidebar1")), FALSE);
 		app->ignore_callback = FALSE;
 	}
-	utils_treeviews_showhide(TRUE);
+	ui_treeviews_show_hide(TRUE);
 	// sets the icon style of the toolbar
 	switch (app->toolbar_icon_style)
 	{
@@ -156,6 +178,7 @@ static void apply_settings(void)
 			break;
 		}
 	}
+	gtk_toolbar_set_style(GTK_TOOLBAR(app->toolbar), app->toolbar_icon_style);
 
 	// sets the icon size of the toolbar, use user preferences (.gtkrc) if not set
 	if (app->toolbar_icon_size == GTK_ICON_SIZE_SMALL_TOOLBAR ||
@@ -163,7 +186,7 @@ static void apply_settings(void)
 	{
 		gtk_toolbar_set_icon_size(GTK_TOOLBAR(app->toolbar), app->toolbar_icon_size);
 	}
-	utils_update_toolbar_icons(app->toolbar_icon_size);
+	ui_update_toolbar_icons(app->toolbar_icon_size);
 
 	// line number and markers margin are by default enabled
 	if (! app->show_markers_margin)
@@ -184,79 +207,69 @@ static void apply_settings(void)
 		gtk_window_set_default_size(GTK_WINDOW(app->window), app->geometry[2], app->geometry[3]);
 	}
 
+	// hide statusbar if desired
+	if (! app->statusbar_visible)
+	{
+		gtk_widget_hide(app->statusbar);
+	}
+
 	app->ignore_callback = TRUE;
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
 				lookup_widget(app->window, "menu_line_breaking1")), app->pref_editor_line_breaking);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
-				lookup_widget(app->window, "menu_use_auto_indention1")), app->pref_editor_use_auto_indention);
+				lookup_widget(app->window, "menu_use_auto_indention1")),
+				(app->pref_editor_indention_mode != INDENT_NONE));
 	app->ignore_callback = FALSE;
 
+	// connect the toolbar dropdown menus
 	gtk_menu_tool_button_set_menu(GTK_MENU_TOOL_BUTTON(
 			lookup_widget(app->window, "menutoolbutton1")), app->new_file_menu);
+	gtk_menu_tool_button_set_menu(GTK_MENU_TOOL_BUTTON(
+			lookup_widget(app->window, "toolbutton9")), app->recent_files_toolbar);
 
 	// set the tab placements of the notebooks
 	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(app->notebook), app->tab_pos_editor);
 	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(msgwindow.notebook), app->tab_pos_msgwin);
 	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(app->treeview_notebook), app->tab_pos_sidebar);
 
-	utils_update_toolbar_items();
+	ui_update_toolbar_items();
+
+	// whether to show notebook tabs or not
+	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(app->notebook), app->show_notebook_tabs);
 }
 
 
 static void main_init(void)
 {
 	// inits
-	app = g_new0(MyApp, 1);
-#ifdef GEANY_DEBUG
-	geany_debug("debug mode built in (can't be disabled)");
-#endif
-	app->debug_mode = debug_mode;
-	geany_debug("GTK+ runtime version: %u.%u.%u",
-		gtk_major_version, gtk_minor_version, gtk_micro_version);
-
-	if (alternate_config)
-	{
-		geany_debug("alternate config: %s", alternate_config);
-		app->configdir = alternate_config;
-	}
-	else
-		app->configdir = g_strconcat(GEANY_HOME_DIR, G_DIR_SEPARATOR_S, ".", PACKAGE, NULL);
 	app->window				= NULL;
-	app->search_text		= NULL;
+	app->project			= NULL;
 	app->open_fontsel		= NULL;
 	app->open_colorsel		= NULL;
 	app->open_filesel		= NULL;
 	app->save_filesel		= NULL;
 	app->prefs_dialog		= NULL;
-	app->find_dialog		= NULL;
-	app->replace_dialog		= NULL;
-	app->find_in_files_dialog = NULL;
 	app->default_tag_tree	= NULL;
 	app->main_window_realized= FALSE;
 	app->tab_order_ltr		= FALSE;
 	app->quitting			= FALSE;
 	app->ignore_callback	= FALSE;
-#ifdef HAVE_FIFO
-	app->ignore_fifo		= ignore_fifo;
-#endif
-#ifdef HAVE_VTE
-	app->lib_vte			= lib_vte;
-#endif
-	app->ignore_global_tags 		= ignore_global_tags;
 	app->tm_workspace				= tm_get_workspace();
 	app->recent_queue				= g_queue_new();
 	app->opening_session_files		= FALSE;
-	html_entities					= NULL;
 
 	app->window = create_window1();
 	app->new_file_menu = gtk_menu_new();
+	app->recent_files_toolbar = gtk_menu_new();
+	app->recent_files_menubar = gtk_menu_new();
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(lookup_widget(app->window, "recent_files1")),
+		app->recent_files_menubar);
 
 	// store important pointers in the MyApp structure
 	app->toolbar = lookup_widget(app->window, "toolbar1");
 	app->tagbar = lookup_widget(app->window, "scrolledwindow2");
 	app->treeview_notebook = lookup_widget(app->window, "notebook3");
 	app->notebook = lookup_widget(app->window, "notebook1");
-	msgwindow.notebook = lookup_widget(app->window, "notebook_info");
 	app->statusbar = lookup_widget(app->window, "statusbar");
 	app->popup_menu = create_edit_menu1();
 	app->toolbar_menu = create_toolbar_popup_menu1();
@@ -279,6 +292,8 @@ static void main_init(void)
 	app->menu_insert_include_item[1] = lookup_widget(app->window, "insert_include2");
 	app->save_buttons[0] = lookup_widget(app->window, "menu_save1");
 	app->save_buttons[1] = lookup_widget(app->window, "toolbutton10");
+	app->save_buttons[2] = lookup_widget(app->window, "menu_save_all1");
+	app->save_buttons[3] = lookup_widget(app->window, "toolbutton22");
 	app->sensitive_buttons[0] = lookup_widget(app->window, "menu_close1");
 	app->sensitive_buttons[1] = lookup_widget(app->window, "toolbutton15");
 	app->sensitive_buttons[2] = lookup_widget(app->window, "menu_change_font1");
@@ -314,15 +329,21 @@ static void main_init(void)
 	app->sensitive_buttons[32] = lookup_widget(app->window, "print1");
 	app->sensitive_buttons[33] = lookup_widget(app->window, "menu_reload_as1");
 	app->sensitive_buttons[34] = lookup_widget(app->window, "menu_select_all1");
+	app->sensitive_buttons[35] = lookup_widget(app->window, "insert_date1");
+	app->sensitive_buttons[36] = lookup_widget(app->window, "menu_format1");
+	app->sensitive_buttons[37] = lookup_widget(app->window, "menu_open_selected_file1");
+	app->sensitive_buttons[38] = lookup_widget(app->window, "menu_insert_special_chars1");
 	app->redo_items[0] = lookup_widget(app->popup_menu, "redo1");
 	app->redo_items[1] = lookup_widget(app->window, "menu_redo2");
 	app->redo_items[2] = lookup_widget(app->window, "toolbutton_redo");
 	app->undo_items[0] = lookup_widget(app->popup_menu, "undo1");
 	app->undo_items[1] = lookup_widget(app->window, "menu_undo2");
 	app->undo_items[2] = lookup_widget(app->window, "toolbutton_undo");
-	msgwindow.tree_status = lookup_widget(app->window, "treeview3");
-	msgwindow.tree_msg = lookup_widget(app->window, "treeview4");
-	msgwindow.tree_compiler = lookup_widget(app->window, "treeview5");
+
+	// set widget names for matching with .gtkrc-2.0
+	gtk_widget_set_name(app->window, "GeanyMainWindow");
+	gtk_widget_set_name(app->toolbar_menu, "GeanyToolbarMenu");
+	gtk_widget_set_name(app->popup_menu, "GeanyEditMenu");
 }
 
 
@@ -338,6 +359,7 @@ gchar *get_argv_filename(const gchar *filename)
 	{
 		//use current dir
 		gchar *cur_dir = g_get_current_dir();
+
 		result = g_strjoin(
 			G_DIR_SEPARATOR_S, cur_dir, filename, NULL);
 		g_free(cur_dir);
@@ -346,141 +368,45 @@ gchar *get_argv_filename(const gchar *filename)
 }
 
 
-#ifdef HAVE_FIFO
-static gboolean read_fifo(GIOChannel *source, GIOCondition condition, gpointer data)
+static void setup_paths()
 {
-	GIOStatus status = G_IO_STATUS_NORMAL;
-	gchar *buffer = NULL;
-	gsize len = 0;
-
-	//status = g_io_channel_read_to_end(source, &buffer, &len, NULL);
-	status = g_io_channel_read_line(source, &buffer, &len, NULL, NULL);
-
-	// try to interpret the received data as a filename, otherwise do nothing
-	if (g_file_test(buffer, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
-	{
-		document_open_file(-1, buffer, 0, FALSE, NULL, NULL);
-		gtk_window_deiconify(GTK_WINDOW(app->window));
-	}
-	else
-	{
-		geany_debug("got data from named pipe, but it does not look like a filename");
-	}
-	g_free(buffer);
-
-	if (status != G_IO_STATUS_NORMAL)
-		return TRUE;
-	else
-	{
-		g_io_channel_unref(app->fifo_ioc);
-		g_io_channel_shutdown(app->fifo_ioc, FALSE, NULL);
-		app->fifo_ioc = g_io_channel_unix_new(open(fifo_name, O_RDONLY | O_NONBLOCK));
-		g_io_add_watch(app->fifo_ioc, G_IO_IN | G_IO_PRI, read_fifo, NULL);
-		return FALSE;
-	}
-}
-
-
-static void write_fifo(gint argc, gchar **argv)
-{
-	gint i;
-	GIOChannel *ioc;
-
-	for(i = 1; i < argc && argv[i] != NULL; i++)
-	{
-		gchar *filename = get_argv_filename(argv[i]);
-
-		if (filename && g_file_test(filename, G_FILE_TEST_IS_REGULAR || G_FILE_TEST_IS_SYMLINK))
-		{
-			ioc = g_io_channel_unix_new(open(fifo_name, O_WRONLY));
-			g_io_channel_write_chars(ioc, filename, -1, NULL, NULL);
-			g_io_channel_flush(ioc, NULL);
-			g_io_channel_shutdown(ioc, TRUE, NULL);
-		}
-		g_free(filename);
-	}
-}
-
-
-/* creates a named pipe in GEANY_FIFO_NAME, to communicate with new instances of Geany
- * to submit filenames and possibly other things */
-static GIOChannel *create_fifo(gint argc, gchar **argv, const gchar *config_dir)
-{
-	struct stat st;
-	gchar *tmp;
-	GIOChannel *ioc;
-
-	tmp = g_strconcat(config_dir, G_DIR_SEPARATOR_S, GEANY_FIFO_NAME, NULL);
-	g_strlcpy(fifo_name, tmp, sizeof(fifo_name));
-	g_free(tmp);
-
-	if (stat(fifo_name, &st) == 0 && (! S_ISFIFO(st.st_mode)))
-	{	// the FIFO file exists, but is not a FIFO
-		unlink(fifo_name);
-	}
-	else if (stat(fifo_name, &st) == 0 && S_ISFIFO(st.st_mode))
-	{	// FIFO exists, there should be a running instance of Geany
-		if (argc > 1)
-		{
-			geany_debug("using running instance of Geany");
-			write_fifo(argc, argv);
-			exit(0);
-		}
-		else
-		{
-			if (dialogs_show_question(_("Geany is exiting because a named pipe was found. Mostly this means, Geany is already running. If you know Geany is not running, you can delete the file and start Geany anyway.\nDelete the named pipe and start Geany?")))
-			{
-				unlink(fifo_name);
-			}
-			else exit(0);
-		}
-	}
-
-	// there is no Geany running, create fifo and start as usual, so we are a kind of server
-	geany_debug("trying to create a new named pipe");
-
-	if ((mkfifo(fifo_name, S_IRUSR | S_IWUSR)) == -1)
-	{
-		if (errno != EEXIST) geany_debug("creating of a named pipe for IPC failed! (%s)", g_strerror(errno));
-		return NULL;
-	}
-
-	ioc = g_io_channel_unix_new(open(fifo_name, O_RDONLY | O_NONBLOCK));
-	g_io_add_watch(ioc, G_IO_IN | G_IO_PRI, read_fifo, NULL);
-
-	return ioc;
-}
-#endif
-
-
-gint main(gint argc, gchar **argv)
-{
-	GError *error = NULL;
-	GOptionContext *context;
-#ifdef HAVE_FIFO
-	GIOChannel *ioc = NULL;
-#endif
-	gint mkdir_result = 0;
-	gint idx;
-	gchar *config_dir;
 	gchar *data_dir;
 	gchar *doc_dir;
-	gchar *locale_dir;
 
 	// set paths
 #ifdef G_OS_WIN32
 	// take the installation directory(the one where geany.exe is located) as the base for the
-	// language catalogs, documentation and data files
+	// documentation and data files
 	gchar *install_dir = g_win32_get_package_installation_directory("geany", NULL);
 
 	data_dir = g_strconcat(install_dir, "\\data", NULL); // e.g. C:\Program Files\geany\data
 	doc_dir = g_strconcat(install_dir, "\\doc", NULL);
-	locale_dir = g_strdup(data_dir);
 
 	g_free(install_dir);
 #else
 	data_dir = g_strdup(PACKAGE_DATA_DIR "/" PACKAGE "/"); // e.g. /usr/share/geany
 	doc_dir = g_strdup(PACKAGE_DATA_DIR "/doc/" PACKAGE "/html/");
+#endif
+
+	// convert path names to locale encoding
+	app->datadir = utils_get_locale_from_utf8(data_dir);
+	app->docdir = utils_get_locale_from_utf8(doc_dir);
+
+	g_free(data_dir);
+	g_free(doc_dir);
+}
+
+
+static void locale_init()
+{
+	gchar *locale_dir = NULL;
+
+#ifdef G_OS_WIN32
+	gchar *install_dir = g_win32_get_package_installation_directory("geany", NULL);
+	// e.g. C:\Program Files\geany\lib\locale
+	locale_dir = g_strconcat(install_dir, "\\lib\\locale", NULL);
+	g_free(install_dir);
+#else
 	locale_dir = g_strdup(PACKAGE_LOCALE_DIR);
 #endif
 
@@ -490,12 +416,24 @@ gint main(gint argc, gchar **argv)
 	textdomain(GETTEXT_PACKAGE);
 #endif
 	g_free(locale_dir);
+}
+
+
+static void parse_command_line_options(gint *argc, gchar ***argv)
+{
+	GOptionContext *context;
+	GError *error = NULL;
+
+	// first initialise cl_options fields with default values
+	cl_options.load_session = TRUE;
+	cl_options.goto_line = -1;
+	cl_options.goto_column = -1;
 
 	context = g_option_context_new(_(" - A fast and lightweight IDE"));
 	g_option_context_add_main_entries(context, entries, GETTEXT_PACKAGE);
 	g_option_group_set_translation_domain(g_option_context_get_main_group(context), GETTEXT_PACKAGE);
 	g_option_context_add_group(context, gtk_get_option_group(TRUE));
-	g_option_context_parse(context, &argc, &argv, &error);
+	g_option_context_parse(context, argc, argv, &error);
 	g_option_context_free(context);
 
 	if (show_version)
@@ -506,21 +444,62 @@ gint main(gint argc, gchar **argv)
 				GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
 		printf("\n");
 
-		return (0);
+		exit(0);
 	}
 
-	gtk_set_locale();
-
-	signal(SIGTERM, signal_cb);
-#ifdef G_OS_UNIX
-	/* ignore SIGPIPE signal for preventing sudden death of program */
-	signal(SIGPIPE, SIG_IGN);
+	app->debug_mode = debug_mode;
+#ifdef GEANY_DEBUG
+	geany_debug("debug mode built in (can't be disabled)");
 #endif
 
-	if (alternate_config) config_dir = g_strdup(alternate_config);
-	else config_dir = g_strconcat(GEANY_HOME_DIR, G_DIR_SEPARATOR_S, ".", PACKAGE, NULL);
+	if (alternate_config)
+	{
+		geany_debug("alternate config: %s", alternate_config);
+		app->configdir = alternate_config;
+	}
+	else
+		app->configdir = g_strconcat(GEANY_HOME_DIR, G_DIR_SEPARATOR_S, ".", PACKAGE, NULL);
 
-	mkdir_result = utils_make_settings_dir(config_dir, data_dir, doc_dir);
+#ifdef GEANY_DEBUG
+	if (generate_datafiles)
+	{
+		filetypes_init_types();
+		configuration_generate_data_files();	// currently only filetype_extensions.conf
+		exit(0);
+	}
+#endif
+	if (generate_tags)
+	{
+		gboolean ret;
+
+		filetypes_init_types();
+		configuration_read_filetype_extensions();	// needed for *.lang.tags filetype matching
+		ret = symbols_generate_global_tags(*argc, *argv);
+		exit(ret);
+	}
+
+
+#ifdef HAVE_SOCKET
+	socket_info.ignore_socket = ignore_socket;
+#endif
+#ifdef HAVE_VTE
+	vte_info.lib_vte = lib_vte;
+#endif
+	app->ignore_global_tags = ignore_global_tags;
+}
+
+
+// Returns 0 if config dir is OK.
+static gint setup_config_dir()
+{
+	gint mkdir_result = 0;
+	gchar *tmp = app->configdir;
+
+	// convert configdir to locale encoding to avoid troubles
+	app->configdir = utils_get_locale_from_utf8(app->configdir);
+	g_free(tmp);
+
+	mkdir_result = utils_make_settings_dir(app->configdir, app->datadir, app->docdir);
 	if (mkdir_result != 0)
 	{
 		if (! dialogs_show_question(
@@ -528,47 +507,169 @@ gint main(gint argc, gchar **argv)
 			  "using Geany without a configuration directory.\nStart Geany anyway?"),
 			  g_strerror(mkdir_result)))
 		{
-			g_free(config_dir);
-			return (0);
+			exit(0);
 		}
 	}
+	return mkdir_result;
+}
 
-#ifdef HAVE_FIFO
-	if (! ignore_fifo) ioc = create_fifo(argc, argv, config_dir);
+
+static void signal_cb(gint sig)
+{
+	if (sig == SIGTERM)
+	{
+		on_exit_clicked(NULL, NULL);
+	}
+}
+
+
+// open files from command line
+static gboolean open_cl_files(gint argc, gchar **argv)
+{
+	gint i;
+
+	if (argc <= 1) return FALSE;
+
+	document_delay_colourise();
+
+	for(i = 1; i < argc; i++)
+	{
+		gchar *filename = get_argv_filename(argv[i]);
+
+		if (filename != NULL &&
+			g_file_test(filename, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
+		{
+			gint idx;
+
+			idx = document_open_file(-1, filename, 0, FALSE, NULL, NULL);
+			// add recent file manually because opening_session_files is set
+			if (DOC_IDX_VALID(idx))
+				ui_add_recent_file(doc_list[idx].file_name);
+		}
+		else if (filename != NULL)
+		{	// create new file if it doesn't exist
+			gint idx;
+
+			idx = document_new_file(filename, NULL);
+			if (DOC_IDX_VALID(idx))
+				ui_add_recent_file(doc_list[idx].file_name);
+		}
+		else
+		{
+			gchar *msg = _("Could not find file '%s'.");
+
+			g_printerr(msg, filename);	// also print to the terminal
+			g_printerr("\n");
+			msgwin_status_add(msg, filename);
+		}
+		g_free(filename);
+	}
+	document_colourise_new();
+	return TRUE;
+}
+
+
+static void load_project_file()
+{
+	gchar *locale_filename;
+
+	g_return_if_fail(project_prefs.session_file != NULL);
+
+	locale_filename = utils_get_locale_from_utf8(project_prefs.session_file);
+
+	if (*locale_filename)
+		project_load_file(locale_filename);
+
+	g_free(locale_filename);
+	g_free(project_prefs.session_file);	// no longer needed
+}
+
+
+gint main(gint argc, gchar **argv)
+{
+	gint idx;
+	gint config_dir_result;
+
+	app = g_new0(MyApp, 1);
+
+	setup_paths();
+	locale_init();
+	parse_command_line_options(&argc, &argv);
+
+	gtk_set_locale();
+
+	signal(SIGTERM, signal_cb);
+#ifdef G_OS_UNIX
+	// SIGQUIT is used to kill spawned children and we get also this signal, so ignore
+	signal(SIGQUIT, SIG_IGN);
+	// ignore SIGPIPE signal for preventing sudden death of program
+	signal(SIGPIPE, SIG_IGN);
 #endif
-	g_free(config_dir);
 
+	config_dir_result = setup_config_dir();
+#ifdef HAVE_SOCKET
+    // check and create (unix domain) socket for remote operation
+	if (! socket_info.ignore_socket)
+	{
+		socket_info.lock_socket = -1;
+		socket_info.lock_socket_tag = 0;
+		socket_info.lock_socket = socket_init(argc, argv);
+		if (socket_info.lock_socket < 0)
+		{
+			// Socket exists
+			if (argc > 1)	// filenames were sent to first instance, so quit
+			{
+				g_free(app->configdir);
+				g_free(app->datadir);
+				g_free(app->docdir);
+				g_free(app);
+				return 0;
+			}
+			// Start a new instance if no command line strings were passed
+			socket_info.ignore_socket = TRUE;
+			cl_options.load_session = FALSE;
+		}
+	}
+#endif
+
+	geany_debug("GTK+ %u.%u.%u, GLib %u.%u.%u",
+		gtk_major_version, gtk_minor_version, gtk_micro_version,
+		glib_major_version, glib_minor_version, glib_micro_version);
 	gtk_init(&argc, &argv);
 
 	// inits
 	main_init();
-	app->datadir = data_dir;
-	app->docdir = doc_dir;
 	gtk_widget_set_size_request(app->window, GEANY_WINDOW_MINIMAL_WIDTH, GEANY_WINDOW_MINIMAL_HEIGHT);
 	gtk_window_set_default_size(GTK_WINDOW(app->window), GEANY_WINDOW_DEFAULT_WIDTH, GEANY_WINDOW_DEFAULT_HEIGHT);
 	encodings_init();
+
 	configuration_load();
-	keybindings_init();
-	notebook_init();
-	templates_init();
-	document_init_doclist();
 	// do this here to let cmdline options overwrite configuration settings
 #ifdef HAVE_VTE
-	app->have_vte = app->load_vte;
-	if (no_vte) app->have_vte = FALSE;
+	vte_info.have_vte = (no_vte) ? FALSE : vte_info.load_vte;
 #endif
-#ifdef HAVE_FIFO
-	app->fifo_ioc = ioc;
-#endif
-	filetypes_init_types();
-	if (generate_datafiles)
-	{
-		configuration_generate_data_files();
-		exit(0);
-	}
+	if (no_msgwin) app->msgwindow_visible = FALSE;
+
+	msgwin_init();
+	search_init();
+	ui_create_insert_menu_items();
+	ui_create_insert_date_menu_items();
+	keybindings_init();
+	tools_create_insert_custom_command_menu_items();
+	notebook_init();
+	filetypes_init();
+	templates_init();
+	document_init_doclist();
 	configuration_read_filetype_extensions();
 
-	gtk_window_set_icon(GTK_WINDOW(app->window), utils_new_pixbuf_from_inline(GEANY_IMAGE_LOGO, FALSE));
+	// set window icon
+	{
+		GdkPixbuf *pb;
+
+		pb = ui_new_pixbuf_from_inline(GEANY_IMAGE_LOGO, FALSE);
+		gtk_window_set_icon(GTK_WINDOW(app->window), pb);
+		g_object_unref(pb);	// free our reference
+	}
 
 	// registering some basic events
 	g_signal_connect(G_OBJECT(app->window), "delete_event", G_CALLBACK(on_exit_clicked), NULL);
@@ -579,79 +680,52 @@ gint main(gint argc, gchar **argv)
 	treeviews_prepare_openfiles();
 	treeviews_create_taglist_popup_menu();
 	treeviews_create_openfiles_popup_menu();
-	msgwin_prepare_status_tree_view();
-	msgwin_prepare_msg_tree_view();
-	msgwin_prepare_compiler_tree_view();
-	msgwindow.popup_status_menu = msgwin_create_message_popup_menu(3);
-	msgwindow.popup_msg_menu = msgwin_create_message_popup_menu(4);
-	msgwindow.popup_compiler_menu = msgwin_create_message_popup_menu(5);
 #ifdef HAVE_VTE
 	vte_init();
 #endif
-	dialogs_create_recent_menu();
-	utils_create_insert_menu_items();
+	ui_create_recent_menu();
 
 	msgwin_status_add(_("This is Geany %s."), VERSION);
-	if (mkdir_result != 0)
-		msgwin_status_add(_("Configuration directory could not be created (%s)."), g_strerror(mkdir_result));
+	if (config_dir_result != 0)
+		msgwin_status_add(_("Configuration directory could not be created (%s)."),
+			g_strerror(config_dir_result));
 
 	// apply all configuration options
 	apply_settings();
 
-	// open files from command line
+	// load any command line files or session files
 	app->opening_session_files = TRUE;
-	if (argc > 1)
+	if (! open_cl_files(argc, argv))
 	{
-		gint i, opened = 0;
-		for(i = 1; i < argc; i++)
+		if (app->pref_main_load_session && cl_options.load_session)
 		{
-			if (argv[i] && g_file_test(argv[i], G_FILE_TEST_IS_REGULAR || G_FILE_TEST_IS_SYMLINK))
+			load_project_file();
+
+			// load session files
+			if (! configuration_open_files())
 			{
-				if (opened < GEANY_MAX_OPEN_FILES)
-				{
-					gchar *filename = get_argv_filename(argv[i]);
-					document_open_file(-1, filename, 0, FALSE, NULL, NULL);
-					g_free(filename);
-					opened++;
-				}
-				else
-				{
-					dialogs_show_error(
-			_("You have opened too many files. There is a limit of %d concurrent open files."),
-			GEANY_MAX_OPEN_FILES);
-					break;
-				}
+				ui_update_popup_copy_items(-1);
+				ui_update_popup_reundo_items(-1);
 			}
 		}
 	}
-	else if (app->pref_main_load_session)
-	{
-		if (! configuration_open_files())
-		{
-			utils_update_popup_copy_items(-1);
-			utils_update_popup_reundo_items(-1);
-		}
-	}
 	app->opening_session_files = FALSE;
 
 	// open a new file if no other file was opened
-	if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook)) == 0)	document_new_file(NULL);
+	if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook)) == 0)	document_new_file(NULL, NULL);
 
-	utils_close_buttons_toggle();
-	utils_save_buttons_toggle(FALSE);
+	ui_close_buttons_toggle();
+	ui_save_buttons_toggle(FALSE);
 
 	idx = document_get_cur_idx();
 	gtk_widget_grab_focus(GTK_WIDGET(doc_list[idx].sci));
 	gtk_tree_model_foreach(GTK_TREE_MODEL(tv.store_openfiles), treeviews_find_node, GINT_TO_POINTER(idx));
-	utils_build_show_hide(idx);
-	utils_update_tag_list(idx, FALSE);
+	build_menu_update(idx);
+	treeviews_update_tag_list(idx, FALSE);
 
 #ifdef G_OS_WIN32
 	// hide "Build" menu item, at least until it is available for Windows
-	gtk_widget_hide(lookup_widget(app->window, "menu_build1"));
 	gtk_widget_hide(app->compile_button);
-	gtk_widget_hide(app->run_button);
-	gtk_widget_hide(lookup_widget(app->window, "separatortoolitem6"));
 	{
 		GtkWidget *compiler_tab;
 		compiler_tab = gtk_notebook_get_tab_label(GTK_NOTEBOOK(msgwindow.notebook),
@@ -666,11 +740,104 @@ gint main(gint argc, gchar **argv)
 
 	configuration_apply_settings();
 
+#ifdef HAVE_SOCKET
+	// register the callback of socket input
+	if (! socket_info.ignore_socket && socket_info.lock_socket > 0)
+	{
+		socket_info.read_ioc = g_io_channel_unix_new(socket_info.lock_socket);
+		socket_info.lock_socket_tag = g_io_add_watch(socket_info.read_ioc,
+						G_IO_IN|G_IO_PRI|G_IO_ERR, socket_lock_input_cb, app->window);
+	}
+#endif
+
 	//g_timeout_add(0, (GSourceFunc)destroyapp, NULL); // useful for start time tests
 	gtk_main();
 	return 0;
 }
 
 
+void main_quit()
+{
+	geany_debug("Quitting...");
 
+#ifdef HAVE_SOCKET
+	socket_finalize();
+#endif
 
+	keybindings_free();
+	filetypes_save_commands();
+	filetypes_free_types();
+	styleset_free_styles();
+	templates_free_templates();
+	msgwin_finalize();
+	search_finalize();
+	build_finalize();
+	document_finalize();
+	symbols_finalize();
+	if (app->project != NULL) project_close();
+
+	tm_workspace_free(TM_WORK_OBJECT(app->tm_workspace));
+	g_free(app->configdir);
+	g_free(app->datadir);
+	g_free(app->docdir);
+	g_free(app->custom_date_format);
+	g_free(app->editor_font);
+	g_free(app->tagbar_font);
+	g_free(app->msgwin_font);
+	g_free(app->long_line_color);
+	g_free(app->context_action_cmd);
+	g_free(app->pref_template_developer);
+	g_free(app->pref_template_company);
+	g_free(app->pref_template_mail);
+	g_free(app->pref_template_initial);
+	g_free(app->pref_template_version);
+	g_free(app->tools_make_cmd);
+	g_free(app->tools_term_cmd);
+	g_free(app->tools_browser_cmd);
+	g_free(app->tools_print_cmd);
+	g_free(app->tools_grep_cmd);
+	g_strfreev(app->custom_commands);
+	while (! g_queue_is_empty(app->recent_queue))
+	{
+		g_free(g_queue_pop_tail(app->recent_queue));
+	}
+	g_queue_free(app->recent_queue);
+
+	if (app->prefs_dialog && GTK_IS_WIDGET(app->prefs_dialog)) gtk_widget_destroy(app->prefs_dialog);
+	if (app->save_filesel && GTK_IS_WIDGET(app->save_filesel)) gtk_widget_destroy(app->save_filesel);
+	if (app->open_filesel && GTK_IS_WIDGET(app->open_filesel)) gtk_widget_destroy(app->open_filesel);
+	if (app->open_fontsel && GTK_IS_WIDGET(app->open_fontsel)) gtk_widget_destroy(app->open_fontsel);
+	if (app->open_colorsel && GTK_IS_WIDGET(app->open_colorsel)) gtk_widget_destroy(app->open_colorsel);
+	if (app->default_tag_tree && GTK_IS_WIDGET(app->default_tag_tree))
+	{
+		g_object_unref(app->default_tag_tree);
+		gtk_widget_destroy(app->default_tag_tree);
+	}
+	scintilla_release_resources();
+#ifdef HAVE_VTE
+	if (vte_info.have_vte) vte_close();
+	g_free(vte_info.lib_vte);
+	g_free(vte_info.dir);
+#endif
+	gtk_widget_destroy(app->window);
+
+	// destroy popup menus
+	if (app->popup_menu && GTK_IS_WIDGET(app->popup_menu))
+					gtk_widget_destroy(app->popup_menu);
+	if (app->toolbar_menu && GTK_IS_WIDGET(app->toolbar_menu))
+					gtk_widget_destroy(app->toolbar_menu);
+	if (tv.popup_taglist && GTK_IS_WIDGET(tv.popup_taglist))
+					gtk_widget_destroy(tv.popup_taglist);
+	if (tv.popup_openfiles && GTK_IS_WIDGET(tv.popup_openfiles))
+					gtk_widget_destroy(tv.popup_openfiles);
+	if (msgwindow.popup_status_menu && GTK_IS_WIDGET(msgwindow.popup_status_menu))
+					gtk_widget_destroy(msgwindow.popup_status_menu);
+	if (msgwindow.popup_msg_menu && GTK_IS_WIDGET(msgwindow.popup_msg_menu))
+					gtk_widget_destroy(msgwindow.popup_msg_menu);
+	if (msgwindow.popup_compiler_menu && GTK_IS_WIDGET(msgwindow.popup_compiler_menu))
+					gtk_widget_destroy(msgwindow.popup_compiler_menu);
+
+	g_free(app);
+
+	gtk_main_quit();
+}
