@@ -1,7 +1,8 @@
 /*
  *      treeviews.c
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,7 +18,11 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: treeviews.c 1291 2007-02-16 16:34:57Z eht16 $
+ * $Id: treeviews.c 1394 2007-03-14 12:11:50Z ntrel $
+ */
+
+/*
+ * Sidebar related code for the Symbol list and Open files GtkTreeViews.
  */
 
 #include <string.h>
@@ -29,6 +34,7 @@
 #include "document.h"
 #include "utils.h"
 #include "ui_utils.h"
+#include "symbols.h"
 
 
 enum
@@ -59,8 +65,8 @@ static gboolean on_treeviews_button_press_event(GtkWidget *widget, GdkEventButto
 
 
 
-/* the following two functions are document-related, but I think they fit better here than in document.c */
-void treeviews_prepare_taglist(GtkWidget *tree, GtkTreeStore *store)
+/* the prepare_* functions are document-related, but I think they fit better here than in document.c */
+static void prepare_taglist(GtkWidget *tree, GtkTreeStore *store)
 {
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
@@ -90,195 +96,70 @@ void treeviews_prepare_taglist(GtkWidget *tree, GtkTreeStore *store)
 }
 
 
-void treeviews_init_tag_list(gint idx)
+static gboolean
+on_default_tag_tree_button_press_event(GtkWidget *widget, GdkEventButton *event,
+		gpointer user_data)
 {
-	// init all GtkTreeIters with -1 to make them invalid to avoid crashes when switching between
-	// filetypes(e.g. config file to Python crashes Geany without this)
-	tv.tag_function.stamp = -1;
-	tv.tag_class.stamp = -1;
-	tv.tag_member.stamp = -1;
-	tv.tag_macro.stamp = -1;
-	tv.tag_variable.stamp = -1;
-	tv.tag_namespace.stamp = -1;
-	tv.tag_struct.stamp = -1;
-	tv.tag_other.stamp = -1;
-
-	switch (doc_list[idx].file_type->id)
+	if (event->button == 3)
 	{
-		case GEANY_FILETYPES_DIFF:
-		{
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_function), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_function), 0, _("Files"), -1);
-			break;
-		}
-		case GEANY_FILETYPES_DOCBOOK:
-		{
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_function), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_function), 0, _("Chapter"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_class), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_class), 0, _("Section"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_member), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_member), 0, _("Sect1"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_macro), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_macro), 0, _("Sect2"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_variable), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_variable), 0, _("Sect3"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_struct), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_struct), 0, _("Appendix"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_other), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_other), 0, _("Other"), -1);
-			//gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_namespace), NULL);
-			//gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_namespace), 0, _("Other"), -1);
-			break;
-		}
-		case GEANY_FILETYPES_LATEX:
-		{
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_function), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_function), 0, _("Command"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_class), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_class), 0, _("Environment"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_member), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_member), 0, _("Section"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_macro), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_macro), 0, _("Subsection"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_variable), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_variable), 0, _("Subsubsection"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_struct), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_struct), 0, _("Label"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_namespace), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_namespace), 0, _("Chapter"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_other), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_other), 0, _("Other"), -1);
-			break;
-		}
-		case GEANY_FILETYPES_PERL:
-		{
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_function), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_function), 0, _("Function"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_class), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_class), 0, _("Package"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_member), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_member), 0, _("My"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_macro), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_macro), 0, _("Local"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_variable), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_variable), 0, _("Our"), -1);
-/*			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_struct), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_struct), 0, _("Label"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_namespace), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_namespace), 0, _("Begin"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_other), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_other), 0, _("Other"), -1);
-*/
-			break;
-		}
-		case GEANY_FILETYPES_RUBY:
-		{
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_function), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_function), 0, _("Methods"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_class), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_class), 0, _("Classes"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_member), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_member), 0, _("Singletons"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_macro), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_macro), 0, _("Mixins"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_variable), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_variable), 0, _("Variables"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_struct), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_struct), 0, _("Members"), -1);
-/*			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_namespace), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_namespace), 0, _("Begin"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_other), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_other), 0, _("Other"), -1);
-*/
-			break;
-		}
-		case GEANY_FILETYPES_PYTHON:
-		{
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_function), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_function), 0, _("Functions"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_class), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_class), 0, _("Classes"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_member), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_member), 0, _("Methods"), -1);
-/*			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_macro), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_macro), 0, _("Mixin"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_variable), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_variable), 0, _("Variables"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_struct), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_struct), 0, _("Members"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_namespace), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_namespace), 0, _("Begin"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_other), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_other), 0, _("Other"), -1);
-*/
-			break;
-		}
-		case GEANY_FILETYPES_JAVA:
-		case GEANY_FILETYPES_D:
-		{
-			// use singular because one file can only belong to one Package / Module
-			gchar *namespace_name = (doc_list[idx].file_type->id == GEANY_FILETYPES_D) ?
-									 _("Module") : _("Package");
+		on_treeviews_button_press_event(widget, event, GINT_TO_POINTER(TREEVIEW_SYMBOL));
+	}
+	return FALSE;
+}
 
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_function), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_function), 0, _("Methods"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_class), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_class), 0, _("Classes"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_struct), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_struct), 0, _("Interfaces"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_member), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_member), 0, _("Members"), -1);
-			//gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_macro), NULL);
-			//gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_macro), 0, _("Macros"), -1);
-			//gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_variable), NULL);
-			//gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_variable), 0, _("Variables"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_namespace), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_namespace), 0, namespace_name, -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_other), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_other), 0, _("Other"), -1);
-			break;
-		}
-		case GEANY_FILETYPES_VHDL:
+
+// update = rescan the tags for document[idx].filename
+void treeviews_update_tag_list(gint idx, gboolean update)
+{
+	if (gtk_bin_get_child(GTK_BIN(app->tagbar)))
+		gtk_container_remove(GTK_CONTAINER(app->tagbar), gtk_bin_get_child(GTK_BIN(app->tagbar)));
+
+	if (app->default_tag_tree == NULL)
+	{
+		GtkScrolledWindow *scrolled_window = GTK_SCROLLED_WINDOW(app->tagbar);
+		GtkWidget *label;
+
+		// default_tag_tree is a GtkViewPort with a GtkLabel inside it
+		app->default_tag_tree = gtk_viewport_new(
+			gtk_scrolled_window_get_hadjustment(scrolled_window),
+			gtk_scrolled_window_get_vadjustment(scrolled_window));
+		label = gtk_label_new(_("No tags found"));
+		gtk_misc_set_alignment(GTK_MISC(label), 0.1, 0.01);
+		gtk_container_add(GTK_CONTAINER(app->default_tag_tree), label);
+		gtk_widget_show_all(app->default_tag_tree);
+		g_signal_connect(G_OBJECT(app->default_tag_tree), "button-press-event",
+			G_CALLBACK(on_default_tag_tree_button_press_event), NULL);
+		g_object_ref((gpointer)app->default_tag_tree);	// to hold it after removing
+	}
+
+	// show default empty tag tree if there are no tags
+	if (idx == -1 || doc_list[idx].file_type == NULL || ! doc_list[idx].file_type->has_tags)
+	{
+		gtk_container_add(GTK_CONTAINER(app->tagbar), app->default_tag_tree);
+		return;
+	}
+
+	if (update)
+	{	// updating the tag list in the left tag window
+		if (doc_list[idx].tag_tree == NULL)
 		{
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_function), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_function), 0, _("Functions"), -1);
-/*			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_class), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_class), 0, _("Constants"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_member), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_member), 0, _("Members"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_macro), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_macro), 0, _("Macros"), -1);
-*/			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_variable), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_variable), 0, _("Variables"), -1);
-/*			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_namespace), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_namespace), 0, _("Namespaces"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_struct), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_struct), 0, _("Signals"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_other), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_other), 0, _("Other"), -1);
-*/
-			break;
-		}
-		default:
-		{
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_function), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_function), 0, _("Functions"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_class), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_class), 0, _("Classes"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_member), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_member), 0, _("Members"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_macro), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_macro), 0, _("Macros"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_variable), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_variable), 0, _("Variables"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_namespace), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_namespace), 0, _("Namespaces"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_struct), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_struct), 0, _("Structs / Typedefs"), -1);
-			gtk_tree_store_append(doc_list[idx].tag_store, &(tv.tag_other), NULL);
-			gtk_tree_store_set(doc_list[idx].tag_store, &(tv.tag_other), 0, _("Other"), -1);
+			doc_list[idx].tag_store = gtk_tree_store_new(1, G_TYPE_STRING);
+			doc_list[idx].tag_tree = gtk_tree_view_new();
+			prepare_taglist(doc_list[idx].tag_tree, doc_list[idx].tag_store);
+			gtk_widget_show(doc_list[idx].tag_tree);
+			g_object_ref((gpointer)doc_list[idx].tag_tree);	// to hold it after removing
 		}
+
+		doc_list[idx].has_tags = symbols_recreate_tag_list(idx);
+	}
+
+	if (doc_list[idx].has_tags)
+	{
+		gtk_container_add(GTK_CONTAINER(app->tagbar), doc_list[idx].tag_tree);
+	}
+	else
+	{
+		gtk_container_add(GTK_CONTAINER(app->tagbar), app->default_tag_tree);
 	}
 }
 
@@ -290,6 +171,7 @@ void treeviews_prepare_openfiles()
 	GtkTreeViewColumn *column;
 	GtkTreeSelection *select;
 	PangoFontDescription *pfd;
+	GtkTreeSortable    *sortable;
 
 	tv.tree_openfiles = lookup_widget(app->window, "treeview6");
 
@@ -311,6 +193,10 @@ void treeviews_prepare_openfiles()
 
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(tv.tree_openfiles), FALSE);
 
+	// sort opened filenames in the store_openfiles treeview
+	sortable = GTK_TREE_SORTABLE(GTK_TREE_MODEL(tv.store_openfiles));
+	gtk_tree_sortable_set_sort_column_id(sortable, 0, GTK_SORT_ASCENDING);
+
 	pfd = pango_font_description_from_string(app->tagbar_font);
 	gtk_widget_modify_font(tv.tree_openfiles, pfd);
 	pango_font_description_free(pfd);
@@ -325,7 +211,8 @@ void treeviews_prepare_openfiles()
 }
 
 
-// Also sets doc_list[idx].iter.
+/* Also sets doc_list[idx].iter.
+ * This is called recursively in treeviews_openfiles_update_all(). */
 void treeviews_openfiles_add(gint idx)
 {
 	GtkTreeIter *iter = &doc_list[idx].iter;
@@ -347,6 +234,7 @@ void treeviews_openfiles_update(gint idx)
 }
 
 
+#if 0
 void treeviews_openfiles_update_all()
 {
 	guint i;
@@ -361,6 +249,7 @@ void treeviews_openfiles_update_all()
 		treeviews_openfiles_add(idx);
 	}
 }
+#endif
 
 
 void treeviews_remove_document(gint idx)
@@ -374,7 +263,7 @@ void treeviews_remove_document(gint idx)
 		gtk_widget_destroy(doc_list[idx].tag_tree);
 		if (GTK_IS_TREE_VIEW(doc_list[idx].tag_tree))
 		{
-			// Because it was ref'd in ui_update_tag_list, it needs unref'ing
+			// Because it was ref'd in treeviews_update_tag_list, it needs unref'ing
 			g_object_unref((gpointer)doc_list[idx].tag_tree);
 		}
 		doc_list[idx].tag_tree = NULL;
@@ -602,7 +491,7 @@ static gboolean on_treeviews_button_press_event(GtkWidget *widget, GdkEventButto
 		{
 			gtk_menu_popup(GTK_MENU(tv.popup_taglist), NULL, NULL, NULL, NULL,
 																event->button, event->time);
-			return TRUE; // prevent selection changed signal for symbol tags
+			return TRUE;	// prevent selection changed signal for symbol tags
 		}
 	}
 	return FALSE;
